@@ -80,6 +80,9 @@ zenity_entry (ZenityData *data, ZenityEntryData *entry_data)
 
   if (data->width > -1 || data->height > -1)
     gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
+
+  if (data->modal)
+    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   
   if (data->ok_label) {
     button = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_entry_ok_button"));
@@ -107,7 +110,7 @@ zenity_entry (ZenityData *data, ZenityEntryData *entry_data)
   n_entries = g_slist_length (entries);
 
   if (n_entries > 1) {
-    entry = gtk_combo_box_new_with_entry ();
+    entry = gtk_combo_box_text_new_with_entry ();
 
     for (tmp = entries; tmp; tmp = tmp->next) {
       gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (entry), tmp->data);
