@@ -35,6 +35,7 @@ static gchar   *zenity_general_window_icon;
 static int      zenity_general_width;
 static int      zenity_general_height;
 static gchar   *zenity_general_dialog_text;
+static gchar   *zenity_general_dialog_icon;
 static gchar   *zenity_general_separator;
 static gboolean zenity_general_multiple;
 static gboolean zenity_general_editable;
@@ -44,6 +45,7 @@ static gboolean zenity_general_dialog_no_markup;
 static gint     zenity_general_timeout_delay;
 static gchar   *zenity_general_ok_button;
 static gchar   *zenity_general_cancel_button;
+static gboolean zenity_general_modal;
 
 /* Calendar Dialog Options */
 static gboolean zenity_calendar_active;
@@ -78,6 +80,7 @@ static gboolean zenity_list_radiolist;
 static gchar   *zenity_list_print_column;
 static gchar   *zenity_list_hide_column;
 static gboolean zenity_list_hide_header;
+static gboolean zenity_list_imagelist;
 
 #ifdef HAVE_LIBNOTIFY
 /* Notification Dialog Options */
@@ -209,6 +212,15 @@ static GOptionEntry general_options[] = {
     N_("Sets the label of the Cancel button"),
     N_("TEXT")
   },
+  {
+    "modal",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_general_modal,
+    "Set the modal hint",
+    NULL
+  },
   {
     NULL
   }
@@ -336,6 +348,15 @@ static GOptionEntry error_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "icon-name",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_dialog_icon,
+    "Set the dialog icon",
+    "ICON-NAME"
+  },
   {
     "no-wrap",
     '\0',
@@ -377,6 +398,15 @@ static GOptionEntry info_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "icon-name",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_dialog_icon,
+    N_("Set the dialog icon"),
+    N_("ICON-NAME")
+  },
   {
     "no-wrap",
     '\0',
@@ -524,6 +554,15 @@ static GOptionEntry list_options[] = {
     N_("Use radio buttons for first column"),
     NULL
   },
+  {
+    "imagelist",
+    '\0',
+    0,
+    G_OPTION_ARG_NONE,
+    &zenity_list_imagelist,
+    N_("Use an image for first column"),
+    NULL
+  },
   {
     "separator",
     '\0',
@@ -711,6 +750,15 @@ static GOptionEntry question_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "icon-name",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_dialog_icon,
+    N_("Set the dialog icon"),
+    N_("ICON-NAME")
+  },
   {
     "no-wrap",
     '\0',
@@ -823,6 +871,15 @@ static GOptionEntry warning_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "icon-name",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_dialog_icon,
+    N_("Set the dialog icon"),
+    N_("ICON-NAME")
+  },
   {
     "no-wrap",
     '\0',
@@ -1277,6 +1334,7 @@ zenity_general_pre_callback (GOptionContext *context,
   zenity_general_dialog_no_wrap = FALSE;
   zenity_general_dialog_no_markup = FALSE;
   zenity_general_timeout_delay = -1;
+  zenity_general_modal = FALSE;
 
   return TRUE;
 }
@@ -1356,6 +1414,7 @@ zenity_list_pre_callback (GOptionContext *context,
   zenity_list_columns = NULL;
   zenity_list_checklist = FALSE;
   zenity_list_radiolist = FALSE;
+  zenity_list_imagelist = FALSE;
   zenity_list_hide_header = FALSE;
   zenity_list_print_column = NULL;
   zenity_list_hide_column = NULL;
@@ -1513,6 +1572,7 @@ zenity_general_post_callback (GOptionContext *context,
   results->data->timeout_delay = zenity_general_timeout_delay;
   results->data->ok_label = zenity_general_ok_button;
   results->data->cancel_label = zenity_general_cancel_button;
+  results->data->modal = zenity_general_modal;
   return TRUE;
 }
 
@@ -1604,6 +1664,7 @@ zenity_error_post_callback (GOptionContext *context,
 
   if (results->mode == MODE_ERROR) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
+    results->msg_data->dialog_icon = zenity_general_dialog_icon;
     results->msg_data->mode = ZENITY_MSG_ERROR; 
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
     results->msg_data->no_markup = zenity_general_dialog_no_markup;
@@ -1622,6 +1683,7 @@ zenity_info_post_callback (GOptionContext *context,
 
   if (results->mode == MODE_INFO) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
+    results->msg_data->dialog_icon = zenity_general_dialog_icon;
     results->msg_data->mode = ZENITY_MSG_INFO; 
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
     results->msg_data->no_markup = zenity_general_dialog_no_markup;
@@ -1687,6 +1749,7 @@ zenity_list_post_callback (GOptionContext *context,
     
     results->tree_data->checkbox = zenity_list_checklist;
     results->tree_data->radiobox = zenity_list_radiolist;
+    results->tree_data->imagebox = zenity_list_imagelist;
     results->tree_data->multi = zenity_general_multiple;
     results->tree_data->editable = zenity_general_editable;
     results->tree_data->print_column = zenity_list_print_column;
@@ -1706,6 +1769,10 @@ zenity_list_post_callback (GOptionContext *context,
       zenity_option_error (zenity_option_get_name (list_options, &zenity_list_radiolist),
                            ERROR_SUPPORT);
 
+    if (zenity_list_imagelist)
+      zenity_option_error (zenity_option_get_name (list_options, &zenity_list_imagelist), 
+                           ERROR_SUPPORT);
+
     if (zenity_list_print_column)
       zenity_option_error (zenity_option_get_name (list_options, &zenity_list_print_column),
                            ERROR_SUPPORT);
@@ -1791,6 +1858,7 @@ zenity_question_post_callback (GOptionContext *context,
   zenity_option_set_dialog_mode (zenity_question_active, MODE_QUESTION);
   if (results->mode == MODE_QUESTION) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
+    results->msg_data->dialog_icon = zenity_general_dialog_icon;
     results->msg_data->mode = ZENITY_MSG_QUESTION;
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
     results->msg_data->no_markup = zenity_general_dialog_no_markup;
@@ -1835,6 +1903,7 @@ zenity_warning_post_callback (GOptionContext *context,
 
   if (results->mode == MODE_WARNING) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
+    results->msg_data->dialog_icon = zenity_general_dialog_icon;
     results->msg_data->mode = ZENITY_MSG_WARNING;
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
     results->msg_data->no_markup = zenity_general_dialog_no_markup;
@@ -1918,7 +1987,9 @@ zenity_forms_post_callback (GOptionContext *context,
         results->forms_data->column_values = g_slist_append (results->forms_data->column_values, values);
         values = zenity_forms_list_values[++i];
       }
-    }
+    } else
+      results->forms_data->column_values = g_slist_append (NULL, "column");
+
     if (zenity_forms_date_format)
       results->forms_data->date_format = zenity_forms_date_format;
     else
