@@ -87,6 +87,44 @@ zenity_tree_toggled_callback (GtkCellRendererToggle *cell, gchar *path_string, g
   gtk_tree_path_free (path);
 }
 
+static void
+zenity_load_pixbuf (GtkTreeViewColumn *tree_column,
+                    GtkCellRenderer *cell,
+                    GtkTreeModel *tree_model,
+                    GtkTreeIter *iter,
+                    gpointer data)
+{
+  static GHashTable *pixbuf_cache = NULL;
+  GError *error = NULL;
+  GdkPixbuf *pixbuf;
+  gchar *str;
+
+  gtk_tree_model_get (tree_model, iter, 0, &str, -1);
+
+  if (!str) 
+    return;
+
+  if (!pixbuf_cache) {
+    pixbuf_cache = g_hash_table_new (g_str_hash, g_str_equal);
+    g_assert(pixbuf_cache);
+  }
+
+  pixbuf = g_hash_table_lookup (pixbuf_cache, str);
+
+  if (!pixbuf) {
+    pixbuf = gdk_pixbuf_new_from_file (str, &error);
+    if (!pixbuf) 
+      g_warning ("Failed to load '%s'", str);
+
+    g_hash_table_insert (pixbuf_cache, g_strdup (str), pixbuf);
+  }
+
+  if (pixbuf) 
+    g_object_set (cell, "pixbuf", pixbuf, NULL);
+
+  g_free (str);
+}
+
 static gboolean
 zenity_tree_handle_stdin (GIOChannel  *channel,
                           GIOCondition condition,
@@ -120,13 +158,13 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 
     string = g_string_new (NULL);
 
-    while (channel->is_readable != TRUE)
+    while ((g_io_channel_get_flags(channel) & G_IO_FLAG_IS_READABLE) != G_IO_FLAG_IS_READABLE)
       ;
     do {
       gint status;
 
       do {
-        if (channel->is_readable == TRUE)
+        if (g_io_channel_get_flags(channel) & G_IO_FLAG_IS_READABLE)
           status = g_io_channel_read_line_string (channel, string, NULL, &error);
         else
           return FALSE;
@@ -249,7 +287,7 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
       GtkWidget *scrolled_window;
       GtkRequisition rectangle;
 
-      gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
+      gtk_widget_get_preferred_size (GTK_WIDGET (tree_view), &rectangle, NULL);
       scrolled_window = GTK_WIDGET (gtk_builder_get_object (builder,
       							 "zenity_tree_window"));
       gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
@@ -329,7 +367,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
     return;
   }
 
-  if (tree_data->checkbox && tree_data->radiobox) {
+  if (tree_data->checkbox + tree_data->radiobox + tree_data->imagebox > 1) {
     g_printerr (_("You should use only one List dialog type.\n")); 
     data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
@@ -345,6 +383,9 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
+  if (data->modal)
+    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+
   if (data->ok_label) {
     button = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_tree_ok_button"));
     gtk_button_set_label (GTK_BUTTON (button), data->ok_label);
@@ -433,6 +474,12 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
         column = gtk_tree_view_column_new_with_attributes (tmp->data,
                                                            cell_renderer, 
                                                            "active", column_index, NULL);
+      } else if (tree_data->imagebox) {
+        GtkCellRenderer *cell_renderer = gtk_cell_renderer_pixbuf_new ();
+        column = gtk_tree_view_column_new_with_attributes (tmp->data,
+                                                           cell_renderer, NULL);
+        gtk_tree_view_column_set_cell_data_func (column, cell_renderer, 
+                                                 zenity_load_pixbuf, NULL, NULL);
       }
       else  {
         if (tree_data->editable) {
@@ -645,7 +692,7 @@ zenity_tree_dialog_response (GtkWidget *widget, int response, gpointer data)
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
       break;
   }
-  if (channel->is_readable == TRUE)
+  if (channel != NULL && g_io_channel_get_flags (channel) & G_IO_FLAG_IS_READABLE)
     g_io_channel_shutdown (channel, TRUE, NULL);
 
   gtk_main_quit ();
@@ -657,10 +704,7 @@ zenity_tree_row_activated (GtkTreeView *tree_view, GtkTreePath *tree_path,
 {
   ZenityData *zen_data = data;
   GtkTreeSelection *selection; 
-  GtkTreeModel *model;
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
-
+  
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
   gtk_tree_selection_selected_foreach (selection, 
                                        (GtkTreeSelectionForeachFunc) zenity_tree_dialog_get_selected, 
