@@ -32,6 +32,7 @@ typedef struct {
   gint   height;
   gint   exit_code;
   gint   timeout_delay;
+  gboolean modal;
 } ZenityData;
 
 typedef enum {
@@ -60,6 +61,7 @@ typedef enum {
 
 typedef struct {
   gchar   *dialog_text;
+  gchar   *dialog_icon;
   MsgMode  mode;
   gboolean no_wrap;
   gboolean no_markup;
@@ -121,6 +123,7 @@ typedef struct {
   gboolean      checkbox;
   gboolean      radiobox;
   gboolean      hide_header;
+  gboolean      imagebox;
   gchar        *separator;
   gboolean      multi;
   gboolean      editable;
